

local tb = {}

local mc = require "movecollide"

local g = love.graphics

tb.state = "LOAD"

local istate = "angle"

local timer = 0
local angle = 270
local speed = 6
local cue = b.create(187, SH / 2 - 5, 0, 0, 15)

local maxPlayers = 2
local turn = 1
local player1Col = -1
local scores = {}
scores[1] = 0

local potted = false
local foul = false

local hs = {}
table.insert(hs, holes.create(2, 33))
table.insert(hs, holes.create(123, 33))
table.insert(hs, holes.create(244, 33))
table.insert(hs, holes.create(2, 148))
table.insert(hs, holes.create(123, 148))
table.insert(hs, holes.create(244, 148))

function tb._getPlayerColour(player)
	if player1Col == -1 then
		return "UNKNOWN"
	end

	if player1Col == 14 then
		if turn == 1 then
			return "YELLOW"
		else
			return "RED"
		end
	else
		if turn == 1 then
			return "RED"
		else
			return "YELLOW"
		end
	end
end

function tb._createBalls()
    tb.setBalls = {}
    table.insert(tb.setBalls, b.create(80 , SH / 2 - 4, 0, 0, 10))
    
    table.insert(tb.setBalls, b.create(72, (SH / 2 - 4) - 4.5, 0, 0, 14))
    table.insert(tb.setBalls, b.create(72, (SH / 2 - 4) + 4.5, 0, 0, 10))

	table.insert(tb.setBalls, b.create(64 , SH / 2 - 4, 0, 0, 0))
	table.insert(tb.setBalls, b.create(64, (SH / 2 - 4) - 8.5, 0, 0, 10))
	table.insert(tb.setBalls, b.create(64, (SH / 2 - 4) + 8.5, 0, 0, 14))

	table.insert(tb.setBalls, b.create(56, (SH / 2 - 4) - 4.5, 0, 0, 10))
	table.insert(tb.setBalls, b.create(56, (SH / 2 - 4) + 4.5, 0, 0, 14))
	table.insert(tb.setBalls, b.create(56, (SH / 2 - 4) - 13, 0, 0, 14))
	table.insert(tb.setBalls, b.create(56, (SH / 2 - 4) + 13, 0, 0, 10))

	table.insert(tb.setBalls, b.create(48 , SH / 2 - 4, 0, 0, 14))
	table.insert(tb.setBalls, b.create(48, (SH / 2 - 4) - 8.5, 0, 0, 14))
	table.insert(tb.setBalls, b.create(48, (SH / 2 - 4) + 8.5, 0, 0, 10))
	table.insert(tb.setBalls, b.create(48, (SH / 2 - 4) - 17, 0, 0, 10))
	table.insert(tb.setBalls, b.create(48, (SH / 2 - 4) + 17, 0, 0, 14))
end

function tb._shuffle()
    for i, ball in ipairs(tb.setBalls) do
       local nidx = RNG:random(#tb.setBalls)
       local tmp = tb.setBalls[nidx]
       tb.setBalls[nidx] = ball
       tb.setBalls[i] = tmp
    end
end

function tb._resetCueBall()
    return b.create(187, SH / 2 - 5, 0, 0, 15)
end

function tb._checkWin()
   for i = 2, #tb.balls do
      if not tb.balls[i].isPotted then
         return false 
      end
   end
   
   return true
end

function tb._load()    
    tb._createBalls()
    tb._shuffle()
    cue = tb._resetCueBall()
    tb.balls = {}
    table.insert(tb.balls, cue)
    for _, ball in ipairs(tb.setBalls) do
       table.insert(tb.balls, ball) 
    end
    tb.state = "INPUT"
end

function tb.reset(p)
	player1Col = -1
    maxPlayers = p
    scores = {}

    for i = 1, maxPlayers do
        scores[i] = 0 
    end
	potted = false
	foul = false
    tb.state = "LOAD"
end

function tb._input() end

function tb._physics()
    local process = true
    local ix = 0
    
    
    while process do
        process = false
        
        for i, ball in ipairs(tb.balls) do
            local s = ball:calcSpeed() - (ix*6)
            if s > 6 then
                process = true
                ball.mv = ball.velocity:normalized() * 6
            elseif s > 0 then
                ball.mv = ball.velocity:normalized() * s
            else
                ball.mv = vector.new(0, 0)
            end
            -- Get movements
            mc.move(ball)
        end
        
        for i, ball in ipairs(tb.balls) do
            -- Get elastic collisions
            mc.checkCollisions(ball, i, tb.balls)
		local wasPotted = ball.isPotted
            mc.checkPotted(ball, hs)
		if(ball.isPotted ~= wasPotted) then
			potted = true
			local c = ball.colour
			local c2 = tb._getPlayerColour(turn)

			if c2 == "RED" and c == 14 then
				foul = true
			elseif c2 == "YELLOW" and c == 10 then
				foul = true
			end

			if c == 14 then --is yellow
				if player1Col == -1 then
					if turn == 1 then
						player1Col = 14
					else
						player1Col = 10
					end
				end
			elseif c == 10 then --is red
				if player1Col == -1 then
					if turn == 1 then
						player1Col = 10
					else
						player1Col = 14
					end
				end
			elseif c == 0 then --is black
				tb.state = "WIN"
				return
			else
				foul = true
			end
		end
            -- Get reflections
            mc.reflect(ball)
            -- Set new positions
            mc.applyMovement(ball)
            -- add friction
            --mc.addFriction(ball, 0.985)
        end
        
        ix = ix + 1
    end

	timer = timer - love.timer.getDelta()
    
    
    if tb._stopPhysicsRQ() then
	if not potted or foul then
		turn = turn == 1 and 2 or 1
	end

        if tb.balls[1].isPotted then
            tb.balls[1] = tb._resetCueBall() 
        end
        
        --[[if tb._checkWin() then
            turn = turn + 1
            if turn > maxPlayers then
                tb.state = "WIN"
            else
                angle = 270
                speed = 6
                tb.state = "LOAD"
            end
        else]]
            tb.state = "INPUT"
        --end
    end
end

function tb._stopPhysicsRQ()
	local maxSpeed = tb._getMaxSpeed()
    return timer <= 0 or maxSpeed == 0
end

function tb._getMaxSpeed()
   local max = 0
   for i, ball in ipairs(tb.balls) do
       local speed = ball.velocity:length()
       if speed > max then
          max = speed 
       end
   end
   
   return max
end

function tb._win()
end

function tb.drawTable()
   g.setColor(palette[12])
   g.rectangle("fill", 0, 32, 256, 128)
   
   g.setColor(palette[6])
   g.rectangle("fill", 0, 32, 256, 6) -- upper limit = 38
   g.rectangle("fill", 0, 154, 256, 6) -- lower limit = 154
   g.rectangle("fill", 0, 32, 6, 128) -- left limit = 6
   g.rectangle("fill", 250, 32, 6, 128) -- right limit = 250
   
   for i, h in ipairs(hs) do
      h:draw() 
   end
end

function tb._isWinner(player, colour)
	--check if winner
	local allPotted = true
	for i, ball in ipairs(tb.balls) do
		if ball.colour == colour then
			if not ball.isPotted then
				allPotted = false
				break
			end
		end
	end

	return allPotted
end

tb.fn = {}
tb.fn.LOAD = tb._load
tb.fn.INPUT = tb._input
tb.fn.PHYSICS = tb._physics
tb.fn.WIN = tb._win

tb.balls = {}

function tb.update()
    tb.fn[tb.state]()
end

function tb.draw()
    tb.drawTable()
    
    for i, ball in ipairs(tb.balls) do
        ball:draw()
    end
    
    if tb.state == "INPUT" then
        for i, ball in ipairs(tb.balls) do
        	local v = ball.velocity:normalized() * 6
		local path = mc.calculatePath(ball, i, v, tb.balls, 0.9)
		if i == 1 then
			v = vector.new(0, -1)
        		v = v:rotated(math.rad(angle)) * speed
			path = mc.calculatePath(tb.balls[1], 1, v, tb.balls, 0.985)
		end

		local cpos = nil
        
		g.setColor(palette[ball.colour])
		g.setLineStyle("rough")
		for j, p in ipairs(path) do
		   if j == 1 then
		       cpos = p
		   else
		      local npos = p
		      g.line(cpos.x, cpos.y, npos.x, npos.y)
		      cpos = npos
		   end
		end
	end
        
        g.setColor(palette[15])
        
        g.print("ANGLE: " .. angle, 8, 8)
        g.print("SPEED: " .. speed, 8, 16)
    end
    
    g.setColor(palette[15])
    local idx = turn > maxPlayers and maxPlayers or turn
    g.print("COLOUR: " .. tb._getPlayerColour(turn), 8, 168)
    g.print("PLAYER " .. turn, 8, 176)
    
    if tb.state == "WIN" then
       g.clear(palette[0])
       g.setColor(palette[15])
       g.print("GAME SET", 8, 8)
	if turn == 1 then
		if tb._isWinner(1, player1Col) then
			g.print("PLAYER 1 WON!!!", 8, 24)
		else
			g.print("PLAYER 2 WON!!!", 8, 24)
		end
	else
		col = player1Col == 14 and 10 or 14
		if tb._isWinner(2, col) then
			g.print("PLAYER 2 WON!!!", 8, 24)
		else
			g.print("PLAYER 1 WON!!!", 8, 24)
		end
	end
       g.print("Play again? (Y/N)", 8, 40)
    end
end

function tb.keypressed(key, scancode, isrepeat)
    if tb.state == "WIN" then
        if key == "y" then
            angle = 270
            speed = 6
            turn = 1
            tb.reset(maxPlayers)
        elseif key == "n" then
           switchExecution("title")
        end
    end
    
    if tb.state == "INPUT" then
        if key == "rctrl" or key == "lctrl" then
           tb.ctrlDown = true 
        end
        
        if key == "rshift" or key == "lshift" then
           tb.shiftDown = true 
        end
        
        if key == "left" then
            if tb.shiftDown then
                angle = angle - 0.1
            elseif tb.ctrlDown then
                angle = angle - 10
            else
                angle = angle - 1
            end
            angle = angle < 0 and (angle + 360) or angle
        elseif key == "right" then
            if tb.shiftDown then
                angle = angle + 0.1
            elseif tb.ctrlDown then
                angle = angle + 10
            else
                angle = angle + 1
            end
            angle = angle > 360 and (angle - 360) or angle
        end
        
        if key == "down" then
            speed = speed - 0.1
            speed = speed < 0.5 and 0.5 or speed
        elseif key == "up" then
            speed = speed + 0.1  
            speed = speed > 12 and 12 or speed
        end
        
        if key == "return" or key == "enter" then
            --local r = (RNG:random() * 2 - 1) * 0.25
            tb.balls[1]:setRotation(math.rad(angle))
            tb.balls[1]:setSpeed(speed)
            istate = "angle"
            scores[turn] = scores[turn] + 1
		timer = 2.5
		potted = false
		foul = false
            tb.state = "PHYSICS" 
        end
    end
end

function tb.keyreleased(key, scancode)
    if tb.state == "INPUT" then
        if key == "rctrl" or key == "lctrl" then
           tb.ctrlDown = false 
        end
        
        if key == "rshift" or key == "lshift" then
           tb.shiftDown = false 
        end
    end
end

return tb
