 
local mc = {}

local UL = 38
local LL = 154
local LEFTL = 6
local RIGHTL = 250

function mc.move(ball)
    ball.collides = false
    --ball.npos = ball.pos + ball.velocity
    ball.npos = ball.pos + ball.mv
end

function mc.applyMovement(ball)
    if ball.isPotted then
        return
    end
    
    ball.pos = ball.npos

	ball.dir = ball.velocity:normalized()
end


function mc.checkCollisions(ball, idx, balls)
    if ball.isPotted then
        return
    end
    
    local offset = vector.new(4, 4)
    local npos = ball.npos
    
    for i, b in ipairs(balls) do
        if i ~= idx and not b.isPotted then
            local off1 = ball:getCentre(true)
            local off2 = b:getCentre(true)
            if mc._collides(off1, off2) then   
                ball.collides = true
                local dist = ball.npos - b.npos
                local xbigger = math.abs(dist.x) >= math.abs(dist.y)
                
                if xbigger then
                    if ball.npos.x < b.npos.x then
                        --left
                        npos.x = b.npos.x - 9
                    elseif ball.npos.x > b.npos.x then
                        --right
                        npos.x = b.npos.x + 9
                    end
                else
                    --below
                    if ball.npos.y < b.npos.y then
                        npos.y = b.npos.y - 9
                    elseif ball.npos.y > b.npos.y then
                        --above
                        npos.y = b.npos.y + 9
                    end
                end

                off1 = npos + offset
                
                local v1 = ball.velocity
                local x1 = off1
                local v2 = b.velocity
                local x2 = off2
                
                local nv1 = v1 - ((vector.dot(v1 - v2, x1 - x2) / math.pow((x1 - x2):length(), 2)) * (x1 - x2))
                local nv2 = v2 - ((vector.dot(v2 - v1, x2 - x1) / math.pow((x2 - x1):length(), 2)) * (x2 - x1))
                
                ball.velocity = nv1
                b.velocity = nv2
            end
        end
    end
    
    if ball.collides then
        ball.npos = npos
    end
end

function mc._collides(b1, b2)
    local d = b1 - b2
    local distance = d:length()
    
    if distance <= 8 then
        return true
    else
        return false
    end
end

function mc.reflect(ball)
    --left right
    if ball.npos.x < LEFTL then
        ball.npos.x = LEFTL
        ball.velocity.x = -ball.velocity.x
    elseif ball.npos.x + 8 > RIGHTL then
        ball.npos.x = RIGHTL - 8
        ball.velocity.x = -ball.velocity.x
    end
    
    if ball.npos.y < UL then
        ball.npos.y = UL
        ball.velocity.y = -ball.velocity.y
    elseif ball.npos.y + 8 > LL then
        ball.npos.y = LL - 8
        ball.velocity.y = -ball.velocity.y
    end
end

function mc.addFriction(ball, factor)
    ball.velocity = ball.velocity * factor
    if ball.velocity:length() < 0.05 then
        ball.velocity = vector.new(0, 0)
    end
end

function mc.checkPotted(ball, hole)
    if ball.isPotted then
        return
    end
    
   for i, h in ipairs(hole) do
       local bx1 = ball.npos.x
       local bx2 = bx1 + 8
       local hx1 = h.x
       local hx2 = h.x + 10
       
       local by1 = ball.npos.y
       local by2 = by1 + 8
       local hy1 = h.y
       local hy2 = h.y + 10
       
       if bx2 > hx1 and bx1 < hx2 and by2 > hy1 and by1 < hy2 then
          ball.isPotted = true
          ball:setSpeed(0)
          return
       end
   end
end

function mc.calculatePath(ball, idx, v, balls, friction)
    local path = {}
    local centre = ball:getCentre()
    local pos = vector.new(centre.x, centre.y)
    
    table.insert(path, pos)
    while v:length() >= 0.05 do
       pos = pos + v
       
       --reflect
       if pos.x - 4 < LEFTL then
            pos.x = LEFTL + 4
            v.x = -v.x
       elseif pos.x + 4 > RIGHTL then
           pos.x = RIGHTL - 4
           v.x = -v.x
       end
       
       if pos.y - 4 < UL then
            pos.y = UL + 4
            v.y = -v.y
       elseif pos.y + 4 > LL then
           pos.y = LL - 4
           v.y = -v.y
       end
       
       table.insert(path, pos)
       
       --check collisions
       for i, bb in ipairs(balls) do
           if i ~= idx then
                local c2 = bb:getCentre()
                if mc._collides(pos, c2) and not bb.isPotted then
                    return path 
                end
           end
        end
        
        v = v * friction
    end
    
    return path
end

return mc
