local vec = {}

local mt = {}

mt.__add = function(a, b)
  local c = vec.new()
  c.x = a.x + b.x
  c.y = a.y + b.y
  return c
end

mt.__sub = function(a, b)
  local c = vec.new()
  c.x = a.x - b.x
  c.y = a.y - b.y
  return c
end

mt.__mul = function(a, b)
  if type(a) == "table" and type(b) == "table" then
    return
  end

  local c = vec.new()
  if type(a) == "table" and type(b) == "number" then
    return mt.__mul(b, a)
  end

  c.x = a * b.x
  c.y = a * b.y
  return c
end

-- should not be use if i am honest
mt.__div = function(a, b)
  if type(a) == "table" and type(b) == "table" then
    return
  end
  if type(a) == "number" and type(b) == "table" then
    return
  end

  local c = vec.new()
  c.x = a.x / b
  c.y = a.y / b
  return c
end

mt.__eq = function(a, b)
  if a.x ~= b.x then
    return false
  end

  if a.y ~= b.y then
    return false
  end

  return true
end

function vec.new(x, y)
  x = x or 0
  y = y or 0

  local v = {}
  v.x = x
  v.y = y

  setmetatable(v, mt)

  v.length = vec.length
  v.normalized = vec.normalized
  v.rotated = vec.rotated
  v.dot = vec.dot
  v.bounceX = vec.bounceX
  v.bounceY = vec.bounceY

  return v
end

function vec.length(v)
  return math.sqrt(math.pow(v.x, 2) + math.pow(v.y, 2))
end

function vec.normalized(v)
  local l = vec.length(v)
  return v / l
end

function vec.rotated(v, th)
  local c = vec.new()

  local tc = math.cos(th)
  local ts = math.sin(th)

  c.x = v.x * tc - v.y * ts
  c.y = v.x * ts + v.y * tc

  return c
end

function vec.dot(a, b)
   return a.x * b.x + a.y * b.y
end

function vec.bounceX(v)
  return vec.new(v.x * -1, v.y)
end

function vec.bounceY(v)
  return vec.new(v.x, v.y * -1)
end

return vec
